#ifndef __RM9K_WDT_H__
#define __RM9K_WDT_H__

/* Device name */
#define WDT_NAME		"wdt_gpi"

/* Resource names */
#define WDT_RESOURCE_REGS	"excite_watchdog_regs"
#define WDT_RESOURCE_IRQ	"excite_watchdog_irq"
#define WDT_RESOURCE_COUNTER	"excite_watchdog_counter"

#endif /* __RM9K_WDT_H__ */
